@@ -48,6 +48,7 @@ public abstract class FurnitureLibraryUserPreferences extends UserPreferences {
     this.editedProperties = new String [] {
         FurnitureLibrary.FURNITURE_ICON_PROPERTY,
         FurnitureLibrary.FURNITURE_NAME_PROPERTY,
+        FurnitureLibrary.FURNITURE_TAGS_PROPERTY,
         FurnitureLibrary.FURNITURE_CATEGORY_PROPERTY,
         FurnitureLibrary.FURNITURE_CREATOR_PROPERTY,
         FurnitureLibrary.FURNITURE_WIDTH_PROPERTY,
@@ -56,6 +57,7 @@ public abstract class FurnitureLibraryUserPreferences extends UserPreferences {
         FurnitureLibrary.FURNITURE_ELEVATION_PROPERTY,
         FurnitureLibrary.FURNITURE_MOVABLE_PROPERTY,
         FurnitureLibrary.FURNITURE_DOOR_OR_WINDOW_PROPERTY,
+        FurnitureLibrary.FURNITURE_DOOR_OR_WINDOW_CUT_OUT_SHAPE_PROPERTY,
         FurnitureLibrary.FURNITURE_STAIRCASE_CUT_OUT_SHAPE_PROPERTY,
         FurnitureLibrary.FURNITURE_MODEL_ROTATION_PROPERTY};
     setUnit(LengthUnit.CENTIMETER);
